/****************************************************************************
* Chemsuite
* Molecular weight parser and Scanner
* chemweight.yy
* (c) 2000 by Ricardo Stefani
* Released under GPL
***************************************************************************/
%{
#include <stdio.h>
#include <string.h>
#include <math.h>
%}
    double mass  = 0;

    char *elementSymbols[] = 
    {
    "H ", "He", "Li", "Be", 
    "B ", "C ", "N ", "O ",
    "F ", "Ne", "Na", "Mg",
    "Al", "Si", "P ", "S ",
    "Cl", "Ar", "K ", "Ca",
    "Sc", "Ti", "V ", "Cr",
    "Mn", "Fe", "Co", "Ni",
    "Cu", "Zn", "Ga", "Ge",
    "As", "Se", "Br", "Kr",
    "Rb", "Sr", "Y ", "Zr",
    "Nb", "Mo", "Tc", "Ru"
    };

    double elementMass[] =
    {
    1.00097, 4.003, 6.941, 9.012,
    10.81, 12.011, 14.007, 15.999,
    18.99, 20.179, 22.99, 24.3050,
    26.982, 28.086, 30.974, 32.06,
    35.453, 39.948, 39.098, 40.080,
    44.956, 47.900, 50.941, 51.996,
    54.938, 55.847, 58.933, 58.700,
    63.546, 65.380, 69.720, 72.590,
    74.922, 78.960, 79.904, 83.800,
    86.468, 83.620, 88.906, 91.220,
    92.906, 95.940, 97.000, 101.07
    };

    
   int Multiply = 1;
   void extractElement(char *, int);   

%%

([A-Z]){1}                   extractElement(yytext, 0); /* H and similar */  
([A-Z]){1}([0-9])+           extractElement(yytext, 1); /* O2 and similar */
([A-Z]){1}([a-z]){1}([0-9])+ extractElement(yytext, 2); /* Mg2 and similar */

"("+([A-Za-z0-9])+")"+([0-9])+ {
                int i, len;

                len = strlen(yytext) - 1;
                while(isdigit(*(yytext+len)))
                        len--;
                len++;
                Multiply = atoi((yytext+len));
                        {
                        char *ptr = (char *)strchr(yytext, ')');

                        while(*ptr != '(')
                                unput(*ptr--);
                        }
                }

        
")"     Multiply = 1;
%%                    

double findMass(char *elem)
 {
 int i = 0;
 while(strcmp(&elementSymbols[i][0],elem))
    i++;
   
 if (&elementSymbols[i][0] == NULL)
    return 0;
else
   return(elementMass[i]);

 }
 
void extractElement(char *str, int caso)
 {
     char elem[3] = "\0\0\0";
     double massVal;
     int sub = 0;

     switch(caso)
        {
        case 0:
         {
          sscanf(str, "%c", &elem[0]);       
          elem[1] = ' ';
          sub = 1;
          break;
         }
        case 1:
         { 
         sscanf(str, "%c%d", &elem[0], &sub);
         elem[1] = ' ';
         break;
         } 
        case 2:
         {
         sscanf(str, "%c%c%d", &elem[0], &elem[1], &sub);
         break; 
         }
       }
     massVal = findMass(elem); 
     mass += Multiply * massVal * sub;

 }           


double getMass()
{
return(mass);
}
